--[[-----------------------------------------------------------------
--	CodeNameBlaze, an ElvUI edit by Blazeflack @ tukui.org
--
--	This file contains changes/additions to the ElvUI Action Bars
-----------------------------------------------------------------]]--
local E, L, V, P, G, _ = unpack(ElvUI); --Import: Engine, Locales, PrivateDB, ProfileDB, GlobalDB, Localize Underscore
local AB = E:GetModule('ActionBars')
local CNB = E:GetModule('CodeNameBlaze')
local LSM = LibStub("LibSharedMedia-3.0")

function CNB:CreateAndModifyBar(id)
	if not E.db.CNB.actionbars.transparentBackdrop then return; end
	--Apply Transparent backdrop
	local bar = _G['ElvUI_Bar'..id]
	if bar.backdrop then
		bar.backdrop:SetTemplate("Transparent")
	end
end
hooksecurefunc(AB, "CreateBar", CNB.CreateAndModifyBar)

function CNB:CreateAndModifyBarPet()
	if not E.db.CNB.actionbars.transparentBackdrop then return; end
	--Apply Transparent backdrop
	local bar = _G['ElvUI_BarPet']
	if bar.backdrop then
		bar.backdrop:SetTemplate("Transparent")
	end
end
hooksecurefunc(AB, "CreateBarPet", CNB.CreateAndModifyBarPet)

function CNB:CreateAndModifyBarShapeShift()
	if not E.db.CNB.actionbars.transparentBackdrop then return; end
	--Apply Transparent backdrop
	local bar = _G['ElvUI_StanceBar']
	if bar.backdrop then
		bar.backdrop:SetTemplate("Transparent")
	end
end
hooksecurefunc(AB, "CreateBarShapeShift", CNB.CreateAndModifyBarShapeShift)

function CNB:StyleButton(button, noBackdrop, adjustChecked)
	--Apply Transparent backdrop
	if not button.noBackdrop and button.backdrop and E.db.CNB.actionbars.transparentButtons then
		button.backdrop:SetTemplate("Transparent")
	end

	--Set macro name font to match AB font
	local name = button:GetName();
	local macroName = _G[name.."Name"];
	if E.db.actionbar.macrotext and button.buttonType ~= "EXTRAACTIONBUTTON" then
		macroName:FontTemplate(LSM:Fetch("font", E.db.actionbar.font), E.db.actionbar.fontSize, E.db.actionbar.fontOutline)
	end
end
hooksecurefunc(AB, "StyleButton", CNB.StyleButton)

function CNB:FixKeybindText(button)
	--Change hotkey text color to pure white and prevent the WoW client from changing it back
	local hotkey = _G[button:GetName()..'HotKey'];
	hotkey:SetVertexColor(1,1,1,1)
	hotkey.SetVertexColor = E.noop
end
hooksecurefunc(AB, "FixKeybindText", CNB.FixKeybindText)

function CNB:PositionBar4AndPetBar()
	if not E:HasMoverBeenMoved('ElvAB_4') and not E:HasMoverBeenMoved('PetAB') and E.db.actionbar.bar4.enabled and E.db.actionbar.barPet.enabled then
		local petAbWidth = PetAB and PetAB:GetWidth() or (E.db.actionbar.barPet.buttonsize + E.db.actionbar.barPet.buttonspacing*2)
		local ab4Width = Elv_AB4 and Elv_AB4:GetWidth() or (E.db.actionbar.bar4.buttonsize + E.db.actionbar.bar4.buttonspacing*2)
		
		if E.myclass == "HUNTER" or E.myclass == "WARLOCK" or (E.myclass == "MAGE" and GetSpecialization() == 3) or (E.myclass == "DEATHKNIGHT" and GetSpecialization() == 3) then --Permanent pets
			PetAB:ClearAllPoints()
			PetAB:SetPoint('RIGHT', E.UIParent, 'RIGHT', -4, 0)
			ElvAB_4:ClearAllPoints()
			ElvAB_4:SetPoint('RIGHT', E.UIParent, 'RIGHT', -(petAbWidth + 6), 0)
		else
			ElvAB_4:ClearAllPoints()
			ElvAB_4:SetPoint('RIGHT', E.UIParent, 'RIGHT', -4, 0)
			PetAB:ClearAllPoints()
			PetAB:SetPoint('RIGHT', E.UIParent, 'RIGHT', -(ab4Width + 6), 0)
		end
	end
end

local f = CreateFrame('Frame')
f:RegisterEvent('PLAYER_ENTERING_WORLD')
f:SetScript('OnEvent', function(self)
	self:UnregisterEvent('PLAYER_ENTERING_WORLD')
	if not E.private.actionbar.enable then return end;
	CNB:PositionBar4AndPetBar()
end)

--ActionBar options
local function ABOptions()
	E.Options.args.CNB.args.config.args.actionbars = {
		order = 1,
		type = 'group',
		name = L["ActionBars"],
		args = {
			header = {
				order = 1,
				type = "header",
				name = CNB:ColorStr(L["ActionBars"]),
			},
			general = {
				order = 2,
				type = 'group',
				name = L["General"],
				guiInline = true,
				args = {
					transparentBackdrop = {
						order = 1,
						type = "toggle",
						name = L["Transparent Backdrop"],
						desc = L["Set ActionBar backdrops to use the transparent template."],
						get = function(info) return E.db.CNB.actionbars.transparentBackdrop end,
						set = function(info, value) E.db.CNB.actionbars.transparentBackdrop = value; E:StaticPopup_Show('CONFIG_RL'); end,
					},
					transparentButtons = {
						order = 2,
						type = "toggle",
						name = L["Transparent Buttons"],
						desc = L["Set ActionBar buttons to use the transparent template."],
						get = function(info) return E.db.CNB.actionbars.transparentButtons end,
						set = function(info, value) E.db.CNB.actionbars.transparentButtons = value; E:StaticPopup_Show('CONFIG_RL'); end,
					},
				},
			},
		},
	}
end
CNB.configs['actionbars'] = ABOptions